/* 
   sitecopy, remote web site copy manager.
   Copyright (C) 1998, Joe Orton <joe@orton.demon.co.uk>
                                                                     
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
  
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/param.h>
#include <stdio.h>

#include "common.h"
#include "sitecopy.h"
#include "ftp.h"
#include "sites.h"

#define RCNAME "/.sitecopyrc"
#define COPYNAME "/.sitecopy/"

/* more kludgy stuff - this returns whether the given action requires
   that a site name is specified in the cmdline */
#define NEEDSITE(x) (x==action_list || x==action_update || \
	(x==action_catchup) || (x==action_resync) || (x==action_init))
/* The maximum number of sites which can be specified on the command 
 * line */
#define MAXSITES 20

enum debug_types debug_level;
enum action_types action;

/* For the site storage */
struct site_t *all_sites;

/* Misc global vars */
char *progname; /* argv[0] */
char *rcfile; /* filename of sitecopyrc */
char *copypath; /* path to sitecopy directory */

char *sitenames[MAXSITES]; /* the sites specified on the command line */
struct site_t *sites[MAXSITES]; /* the sites info */
int numsites; /* the number of sites specified */

/* Misc options */
bool prompting; /* Do we want confirmations? */

void init( int argc, char *argv[] ) {
    char *home;
    int optc;
    bool allsites;
    extern char *optarg;
    extern int optind;

    /* Defaults */
    action = action_list;
    allsites = false;
    prompting = false;
    debug_level = debug_none;
    progname = argv[0];

    /* Read the cmdline args */
    while( (optc = getopt( argc, argv, "d:p:r:hucilvy" ) ) != -1) {
	switch( optc ) {
	case 'l': action = action_list; break;
	case 's': action = action_resync; break;
	case 'r': 
	    if( strlen(optarg) != 0 ) {
		rcfile = strdup( optarg );
	    } else {
		usage( );
		exit(-1);
	    }
	    break;
	case 'p':
	    if( strlen(optarg) != 0 ) {
		if( optarg[strlen(optarg)] != '/' ) {
		    copypath = malloc( strlen(optarg) + 2 );
		    strcpy( copypath, optarg );
		    strcat( copypath, "/" );
		} else {
		    copypath = strdup( optarg );
		}
	    } else {
		usage( );
		exit(-1);
	    }
	    break;
	case 'i': action = action_init; break;
	case 'c': action = action_catchup; break;
	case 'u': action = action_update; break;
	case 'v': action = action_view; break;
	case 'd': /* set debugging level */
	    switch( atoi(optarg) ) {
	    case 0: 
		debug_level = debug_none;
		printf( "Debugging off.\n" );
		break;
	    case 1:
		debug_level = debug_some;
		printf( "Debugging some.\n" );
		break;
	    case 2: 
		debug_level = debug_full; 
		printf( "Debugging full.\n" );
		break;
	    default:
		usage( ); exit(-1);
	    }
	    break;
	case 'y':
	    prompting = true;
	    printf( "Prompting on.\n" );
	    break;
	case 'a':
	    allsites = true;
	    break;
	case 'h': usage( ); exit(-1);
	case '?': usage( ); exit(-1);
	}
    }

    /* Get those site names off the end of the cmdline */
    for( numsites = 0 ; optind < argc; optind++ ) {
	sitenames[numsites++] = argv[optind];
	if( numsites == MAXSITES ) {
	    printf( "%s: Warning: Only %d sites can be specified on the command line!\nExtra entries are being skipped.\n", progname, MAXSITES );
	    break;
	}
    }

    /* Assign default filenames if they didn't give us any */
    home = getenv("HOME");
    if( rcfile == NULL ) {
	rcfile = (char *) malloc( strlen(RCNAME) + strlen(home) + 1);
	strcpy( rcfile, home );
	strcat( rcfile, RCNAME );
    }
    if( copypath == NULL ) {
	copypath = (char *) malloc( strlen(COPYNAME) + strlen(home) + 1);
	strcpy( copypath, home );
	strcat( copypath, COPYNAME );
    }
}

/* Prompts for confirmation - returns */
bool prompt( ) {
    char in[BUFSIZ];
    printf( "? (y/n) " );
    fgets( in, BUFSIZ, stdin );
    return (*in == 'y'); /* only return true if they put a 'y' first */
}

int main( int argc, char *argv[] ) {
    int ret, num, numgoodsites;
    struct site_t *the_site;

    /* Initialize up */
    ret = 0;
    init( argc, argv );

    /* Read the rcfile */
    if( site_readsites( rcfile, &all_sites) ) {
	fprintf( stderr, "%s: Error: Could not read rcfile: %s.\n",
		 progname, rcfile );
	exit(-1);
    }

    if( NEEDSITE( action ) ) {
	/* Find the site they want */
	numgoodsites = numsites;

	for( num=0; num < numsites; num++ ) {
	    if ( site_find( &all_sites, &sites[num], sitenames[num]) ) {
		printf( "%s: Error: No site called `%s' found - skipping.\n", 
			progname, sitenames[num] );
		sites[num] = NULL;
		numgoodsites--;
	    } else if( site_readfiles( sites[num] ) ) {
		printf( "%s: Error: Could not read site info file for `%s'.\n",
			progname, sitenames[num] );
		numgoodsites--;
	    }
	}
	
	if( numgoodsites==0 ) {
	    printf( "%s: No sites specified.\n", progname );
	    exit(-1);
	}
		
    }

    switch( action ) {
    case action_update:
	for( num = 0; num < numsites; num++ ) {
	    if( (the_site = sites[num]) != NULL ) {
		printf( "%s: Site `%s' (%s:%s%s) in %s\n", 
			progname, the_site->name, the_site->server, 
			DIRPREFIX(the_site->remote_dir), the_site->remote_dir,
			the_site->local_dir );
		if( the_site->has_changed ) {
		    ret = site_update( the_site );
		} else {
		    printf( "%s: Nothing to update - no changes found.\n", progname );
		    ret = 0;
		}
	    }
	}
	break;
    case action_list: 
	for( num = 0; num < numsites; num++ ) {
	    if( (the_site = sites[num]) != NULL ) {
		printf( "%s: Site `%s' (%s:%s%s) in %s\n", 
			progname, the_site->name, the_site->server, 
			DIRPREFIX(the_site->remote_dir), the_site->remote_dir,
			the_site->local_dir );
		site_list( the_site );
		if( the_site->has_changed ) {
		    printf( "%s: The remote site needs updating.\n", progname);
		    ret = 1;
		} else {
		    printf( "%s: The remote site does not need updating.\n", progname );
		    ret = 0;
		}
	    }
	}
	break;
    case action_view:
	site_dumpsites( all_sites );
	break;
    case action_init:
	for( num = 0; num < numsites; num++ ) {
	    if( (the_site = sites[num]) != NULL ) {
		printf( "%s: Site `%s' (%s:%s%s) in %s\n", 
			progname, the_site->name, the_site->server, 
			DIRPREFIX(the_site->remote_dir), the_site->remote_dir,
			the_site->local_dir );
		printf("Initializing site... ");
		the_site->files = NULL; /* A `Bit' of a hack */
		site_writefiles( the_site );
		printf( "done.\nAll the files and directories are marked as NOT updated remotely.\n" );
	    }
	}
	break;
    case action_catchup:
	for( num = 0; num < numsites; num++ ) {
	    if( (the_site = sites[num]) != NULL ) {
		printf( "%s: Site `%s' (%s:%s) in %s\n", 
			progname, the_site->name, the_site->server, 
			the_site->remote_dir, the_site->local_dir );
		site_catchup( the_site );
	    }
	}
	break;
    default:
	printf("Unimplemented - sorry.\n");
	ret = -1;
	break;
    }
    return ret;
}

void usage( ) {
    fprintf( stderr,
"Usage: %s [options] sitename\n\
Options: [-d x] {-h|-v|-l|-i|-c|-u} [-y] [-r fn] [-p dir]\n\
\n\
\t-d\tSet debugging level\n\
\t-r\tSpecify alternate run control file\n\
\t-p\tSpecify alternate site storage directory\n\
\t-y\tRequest confirmation before making each update\n\
\t-h\tDisplay this help message\n\
\t-v\tList the site definitions\n\
\t-l\tList files which have been added, changed, moved and deleted\n\
\t-i\tInitialize - mark all files and directories as not updated\n\
\t-c\tCatch up - mark all files and directories as updated\n\
\t-u\tUpdate the remote site\n\n\
I am sitecopy version %s\n\n", progname, VERSION );

/* \t-a\tPerform the given action on ALL sites defined\n\
   \t-r\tResyncronize the remote site list\n\
*/

}

