/* 
   sitecopy, remote web site copy manager.
   Copyright (C) 1998, Joe Orton <joe@orton.demon.co.uk>
                                                                     
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
  
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <time.h>
#include <ftw.h> 

#include "common.h"

#define DIRPREFIX(item) (item[0]!='/')?"~/":""

struct site_file_t;
struct site_t;

enum file_diff {
    file_unchanged, /* Remote file is same as local file */
    file_changed, /* File has changed locally, and should be uploaded */
    file_new, /* File is new locally, and should be uploaded */
    file_deleted,  /* File deleted locally, and should be deleted remotely */
    file_moved /* File has been moved locally, should be moved remotely */
};

enum site_perm_modes {
    sitep_ignore, /* Ignore file permissions */
    sitep_exec, /* Maintain execute permissions */
    sitep_all /* Maintain all permissions */
};

struct site_file_t {
    enum file_diff diff;
    bool dir; /* This is a directory */
    char *name; /* the base filename name without the path */
    char *filename; /* the file name relative to site root */
    char *remotename; /* the full remote filename */
    char *localname; /* the full local filename */
    
    struct site_file_t *old; /* the item before it was moved */

    time_t localtime; /* the local last mod time of the file */
    time_t remotetime; /* the remote last mod time of the file */
    off_t localsize; /* the local file size */
    off_t remotesize; /* the remote file size */
    mode_t mode; /* the permissions */
    bool updated; /* whether the remote file has been updated according
		   * to the diff setting */

    /* Linked list nodes */
    struct site_file_t *next;
    struct site_file_t *prev;

};

/* This represents a site */
struct site_t {
    char *name; /* symbolic name for site */
    char *server; /* server name */
    char *username; /* username to log in to server */
    char *password; /* password for ditto */
    char *remote_dir; /* root directory of site on server */
    char *local_dir; /* root directory of site locally */
    char *infofile;  /* local storage file in ~/.sitecopy/  */
    /* Options for the site */
    enum site_perm_modes perms; /* permissions maintenance mode */
    bool nodelete;
    bool checkmoved;
    
    struct site_file_t *files; /* list of new files */

    /* Some useful counts for the files */
    int numnew; /* number of new files */
    int numchanged; /* number of changed files */
    int numdeleted; /* number of deleted files */
    int nummoved; /* number of moved files */
    int numunchanged; /* number of unchanged files */

    bool has_changed; /* whether the site has actually changed */

    struct site_t *next;

};

/* site_readsites will read ~/.sitecopyrc and fill the sites list */
int site_readsites( char *filename, struct site_t **sites );

void site_assignnames( struct site_t *the_site, struct site_file_t *the_file );

/* This reads off the local files from the disk, and the
 * remote files from the remote files file */
int site_readfiles( struct site_t * );

/* This marks remote files as updated locally */
int site_catchup( struct site_t * );

/* This writes the remote files list back to the file */
int site_writefiles( struct site_t * );

/* Updates the remote files list of the given site.
 * Tricky, this - have to do a load of LIST's, and try and
 * parse the response sensibly */
int site_update( struct site_t * );

/* Try and find a site of given name. Returns non-zero
 * if a site is found, and points the_site to it. Else 
 * returns zero */
int site_find( struct site_t **, struct site_t **the_site, char *sitename );

int site_verify( struct site_t *any_site );

void site_checkmoved( struct site_t *any_site );

int site_list( struct site_t *the_site );

/* Dumps all the info about the given sites */
void site_dumpsites( struct site_t *the_sites );

/* Syncronizes the remote site with the local copy.
 * This is the core functionality of sitecopy. */
int site_resync( struct site_t *the_site );

#ifdef FTWCONST
int ftw_callback( const char *, struct stat *, int );
#else
int ftw_callback( const char *, const struct stat *, int );
#endif
